var { execute } = require('../src/index');
const fs = require('fs');


let _list = []
const writeFileRecursive = function (path, buffer, callback) {
    let lastPath = path.substring(0, path.lastIndexOf("/"));
    fs.mkdir(lastPath, { recursive: true }, (err) => {
        if (err) return callback(err);
        fs.writeFile(path, buffer, function (err) {
            if (err) return callback(err);
            return callback(null);
        });
    });
}
/**
 * 编译gui
 * @param {*} psdPath psd路径
 * @param {*} outPath gui输出路径
 */
 exports.parseGui = async (psdPath,outPath) => {
    _list = []
    let result = await execute(psdPath, { mode: 'none', singleView: true, projectPath: "./" })
    const buffer = formateXml(`<?xml version="1.0"?><root>` + renderTreeNodes([JSON.parse(result).view]) + `</root>`);
    writeFileRecursive(outPath + ".xml", buffer, (err) => {
        if (err) console.error(err);
        console.info(outPath + ".xml" + " success"["green"]);
    });
    writeJs(_list,outPath)
}


function writeJs(assets,outPath) {
let _str = `
//respath
let res="./";
`
   // console.log(assets)
    for (let i = 0; i < assets.length; i++) {
        _str += `this.guiSources['${assets[i]}']=res+"${assets[i]}.png;"
`
    }
    const buffer = _str;
    writeFileRecursive(outPath + ".js", buffer, (err) => {
        if (err) console.error(err);
        console.info(outPath + ".js" + " success"["green"]);
    });
}


function renderTreeNodes(data) {
    if (data[0] !== null) {
        return data.map((item) => {
            if (!item) {
                return '';
            }
            if (item.children) {
                return `<${item.nodeType} name="${item.name}" thickness="0" verticalAlignment="Control.HORIZONTAL_ALIGNMENT_TOP" horizontalAlignment="Control.HORIZONTAL_ALIGNMENT_LEFT" ${appendProps(item)}>${renderTreeNodes(item.children)}</${item.nodeType}>`;
            }
            return `<${item.nodeType} name="${item.name}"  thickness="0" verticalAlignment="Control.HORIZONTAL_ALIGNMENT_TOP" horizontalAlignment="Control.HORIZONTAL_ALIGNMENT_LEFT" ${appendProps(item)}></${item.nodeType}>`
        }).join("");
    }
}

function isNumber(val) {
    var regPos = /^\d+(\.\d+)?$/; //非负浮点数
    var regNeg = /^(-(([0-9]+\.[0-9]*[1-9][0-9]*)|([0-9]*[1-9][0-9]*\.[0-9]+)|([0-9]*[1-9][0-9]*)))$/; //负浮点数
    if (regPos.test(val) || regNeg.test(val)) {
        return true;
    } else {
        return false;
    }
}

//格式化xml代码
function formateXml(xmlStr) {
    text = xmlStr;
    //使用replace去空格
    text = '\n' + text.replace(/(<\w+)(\s.*?>)/g, function ($0, name, props) {
        return name + ' ' + props.replace(/\s+(\w+=)/g, " $1");
    }).replace(/>\s*?</g, ">\n<");
    //处理注释
    text = text.replace(/\n/g, '\r').replace(/<!--(.+?)-->/g, function ($0, text) {
        var ret = '<!--' + escape(text) + '-->';
        return ret;
    }).replace(/\r/g, '\n');
    //调整格式  以压栈方式递归调整缩进
    var rgx = /\n(<(([^\?]).+?)(?:\s|\s*?>|\s*?(\/)>)(?:.*?(?:(?:(\/)>)|(?:<(\/)\2>)))?)/mg;
    var nodeStack = [];
    var output = text.replace(rgx, function ($0, all, name, isBegin, isCloseFull1, isCloseFull2, isFull1, isFull2) {
        var isClosed = (isCloseFull1 == '/') || (isCloseFull2 == '/') || (isFull1 == '/') || (isFull2 == '/');
        var prefix = '';
        if (isBegin == '!') {//!开头
            prefix = setPrefix(nodeStack.length);
        } else {
            if (isBegin != '/') {///开头
                prefix = setPrefix(nodeStack.length);
                if (!isClosed) {//非关闭标签
                    nodeStack.push(name);
                }
            } else {
                nodeStack.pop();//弹栈
                prefix = setPrefix(nodeStack.length);
            }
        }
        var ret = '\n' + prefix + all;
        return ret;
    });
    var prefixSpace = -1;
    var outputText = output.substring(1);
    //还原注释内容
    outputText = outputText.replace(/\n/g, '\r').replace(/(\s*)<!--(.+?)-->/g, function ($0, prefix, text) {
        if (prefix.charAt(0) == '\r')
            prefix = prefix.substring(1);
        text = unescape(text).replace(/\r/g, '\n');
        var ret = '\n' + prefix + '<!--' + text.replace(/^\s*/mg, prefix) + '-->';
        return ret;
    });
    outputText = outputText.replace(/\s+$/g, '').replace(/\r/g, '\r\n');
    return outputText;
}

//计算头函数 用来缩进
function setPrefix(prefixIndex) {
    var result = '';
    var span = '    ';//缩进长度
    var output = [];
    for (var i = 0; i < prefixIndex; ++i) {
        output.push(span);
    }
    result = output.join('');
    return result;
}

function appendProps(item) {
    let props = ""
    if (item.properties && item.properties.style) {
        for (let type in item.properties.style) {
            props += ` ${type}="${item.properties.style[type]}" `
        }
    }
    if (item.properties && item.properties.attrs) {
        for (let type in item.properties.attrs) {
            if (type == "text") {
                let text = item.properties.attrs[type].toString().replace(/[\r\n]/g, "")
                if (isNumber(text)) {
                    text = "_" + text
                }
                props += ` ${type}="${text}" `
            } else if (type == "source") {
                let _value = "{{guiSources." + item.properties.attrs[type].replace("asset://", "") + "}}"
                _list.push(item.properties.attrs[type].replace("asset://", ""))
                props += ` ${type}="${_value}" `
            }
        }
    }
    return props
}